##############################################################################
#
# PhotoListings for Zope
#
# Copyright (c) Mark Gibson  All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Mark Gibson      mngibso@excite.com
#
##########################################################################
__doc__ = """
   JobListings Product
      JobListings is designed for use with Zope.  http://zope.org

   file: JobEntry.py
   
   JobEntry.py defines the JobEntry class.  JobEntry represents a single
   employement add.

"""
from AccessControl import ClassSecurityInfo
from Globals import DTMLFile
from OFS import SimpleItem, PropertyManager
import Globals
from Products.ZCatalog.CatalogAwareness import CatalogAware
from DateTime import DateTime
from AccessControl import SecurityManagement

__version__ = '0.1'

manage_addJobEntryForm = DTMLFile('dtml/addJobEntry',globals())

job_entry_index_html = DTMLFile('dtml/jobEntryView',globals())

def manage_addJobEntry(self, id, title='', REQUEST=None):
   """ Create a new Job Entry """
   if SecurityManagement.getSecurityManager().getUser().has_permission('Add Job Entry Without Review',self):
      status='Published'
   else:
      status='Private'
   job=JobEntry(id,title, status, REQUEST)
   self._setObject(job.getId(),job)

   #If this is being added inside of a Job Listings, set the catalog
   for jobListings in self.superValues('Job Listings'):
      job.manage_editCataloger(jobListings.catalog_id)
      break
   job.index_object()
   if REQUEST is not None:
      if REQUEST.get('submit','') == ' Add and Edit ':    
         REQUEST['RESPONSE'].redirect(self.absolute_url()+ '/' + job.getId() + '/manage_propertiesForm')
      else:
         REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

class JobEntry(CatalogAware, SimpleItem.SimpleItem, PropertyManager.PropertyManager):
   """ JobEntry class:  """
   meta_type='Job Entry'
   index_html=job_entry_index_html

   security=ClassSecurityInfo()
   states_list = ('','AL','AK','AZ','AR','CA','CO','CT','DE','DC','FL','GA','GU',
     'HI','HI','ID','IL','IN','IA','KS','KY','LA','ME','MD','MA','MI','MN',
     'MS','MO','MT','NE','NV','NH','NJ','NM','NY','NC','ND','OH','OK','OR',
      'PA','PR','RI','SC','SD','TN','TX','UT','VT','VA','VI','WA','WV','WI',
      'WY')
   status_list = ('Published','Private') 
   _properties=(
                {'id':'publishDate', 'type': 'date', 'mode': 'w'},
                {'id':'status', 'type': 'selection', 'mode': 'w',
                       'select_variable': 'status_list'},
                {'id':'title', 'type': 'string', 'mode': 'w'},
                {'id':'jobType', 'type': 'multiple selection', 'mode': 'w',
                        'select_variable': 'job_types'   },
                {'id':'description', 'type': 'text', 'mode': 'w'},
                {'id':'deadlineDate', 'type': 'date', 'mode': 'w'},
                {'id':'jobCity', 'type': 'string', 'mode': 'w'},
                {'id':'jobState', 'type': 'selection', 'mode': 'w', 
                         'select_variable': 'states_list' },
                {'id':'organization', 'type': 'string', 'mode': 'w'},
                {'id':'orgAddress', 'type': 'text', 'mode': 'w'},
                {'id':'orgCity', 'type': 'string', 'mode': 'w'},
                {'id':'orgState', 'type': 'selection', 'mode': 'w', 
                         'select_variable': 'states_list' },
                {'id':'orgPostalCode', 'type': 'string', 'mode': 'w'},
                {'id':'orgUrl', 'type': 'string', 'mode': 'w'},
                {'id':'requirements', 'type': 'text', 'mode': 'w'},
                {'id':'validDate', 'type': 'date', 'mode': 'w'},
                {'id':'expirationDate', 'type': 'date', 'mode': 'w'},
                {'id':'contactInformation', 'type': 'text', 'mode': 'w'},

               )

   #initialize properties
   publishDate=DateTime()
   title=''
   jobType=()
   description=''
   deadlineDate=DateTime()
   jobCity=''
   jobState=''
   organization=''
   orgAddress=''
   orgCity=''
   orgState=''
   orgPostalCode=''
   orgUrl=''
   requirements=''
   validDate=DateTime()
   expirationDate=DateTime()+180
   contactInformation=''
   status='Published'
   default_catalog='Catalog'

   manage_options = (
   {'label':'Properties', 'action':'manage_propertiesForm',
         },
   {'label':'View', 'action':'index_html',
         'help':('OFSP','Properties.stx')},
 
   ) + SimpleItem.SimpleItem.manage_options 

   def __init__(self, id, title='', status='Private', REQUEST=None):
      """ """
      self.id=id
      self.title=title
      #set the catalog for this object from a REQUEST variable, or
      #default to 'Catalog'
      self.status=status

      if REQUEST:
         self.manage_editCataloger(REQUEST.get('cataloger',self.default_catalog))

   def fmtCSZ(self, city='', state='', zip=None):
      """ Formats City, State  Zip"""
      ret=''
      if city or state:
         ret='%s%s  '%(city, state)
      if city and state:
         ret='%s, %s  '%(city,state)
      if zip:
         ret=ret+zip
      return ret 

   security.declareProtected('reviewJob','Add Job Entry Without Review')
   def reviewJob(self):
      """ """
      pass

   def manage_changeProperties(self, REQUEST):
      """ manage_changeProperties """
      ret = PropertyManager.PropertyManager.manage_changeProperties(self,REQUEST)
      self.reindex_object()
      return ret 

   def manage_editProperties(self, REQUEST):
      """ manage_editProperties """
      ret = PropertyManager.PropertyManager.manage_editProperties(self,REQUEST)
      self.reindex_object()
      return ret 

   security.declareProtected('View','getFormattedContactAddress')
   def getFormattedContactAddress(self):
      """  
          format and return a contact address string as:
          address
          city, state  zip
      """
      return self.getFormattedAddress(self.contactAddress, self.contactCity, 
            self.contactState, self.contactZip)

   def PrincipiaSearchSource(self):
      """ returns relevant text for searches """
      ret =  self.description + self.title + self.jobOrganization + \
             self.contactOrganization + self.contactText
      for str in self.jobType:
         ret=ret + ' ' + str
      return ret

   security.declareProtected('View','getFormattedJobAddress')
   def getFormattedJobAddress(self):
      """  
          format and return a job address string as:
          address
          city, state  zip
      """
      return self.getFormattedAddress(self.jobAddress, self.jobCity, 
            self.jobState, self.jobZip)

   security.declareProtected('View','getFormattedAddress')
   def getFormattedAddress(self, address, city, state, zip ):
      """  
          format and return an address string as:
          address
          city, state  zip
      """
      outString=''
      if address:
         outString=outString + address 
         if city or state or zip:
            outString=outString+'<br />'
      if city or state or zip:
         if city and state:
            outString=outString + city + ', ' + state + '  '
         elif city:
            outString=outString + city + '  '
         elif state:
            outString=outString + state + '  '
         if zip:
            outString=outString+zip 
      return outString

   def changeProperties(self, REQUEST=None, **kw):
      """
      Copied from PropertyManager class.  Only change is 
      we don't want to return anything.

      Change existing object properties.

      Change object properties by passing either a mapping object
      of name:value pairs {'foo':6} or passing name=value parameters
      """
      if REQUEST is None:
          props={}
      else: props=REQUEST
      if kw:
          for name, value in kw.items():
              props[name]=value
      propdict=self.propdict()
      for name, value in props.items():
          if self.hasProperty(name):
              self._updateProperty(name, value)

      self.reindex_object()

   def stateTransition(self, new_state, force=0):
      """ Do something when transitioning to new state """
     
      if not force and new_state==self.status:
         return 
      if(new_state == 'Published'):
         #Make the View permission Acquired
         self.publishDate=DateTime()
         self.manage_permission('View',[],acquire=1)  
         self.manage_permission('Access contents information',[],acquire=1)  
      if(new_state == 'Private'):
         #Restrict View to Managers and Owners
         self.manage_permission('View',['Manager','Owner'],acquire=0)  
         self.manage_permission('Access contents information',['Manager','Owner'],acquire=0)  

   def _updateProperty(self, name, value):
      """Update property value by name. """

      if name=='status' and value != self.status:
         #do the state transition if the status has changed
         self.stateTransition( value )
      PropertyManager.PropertyManager._updateProperty(self,name,value)

